local DEG_ADDON = _G["DEG_CURRENT_ADDON"]

local function d(msg)
  _G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT]:d(msg)
end

local function ts(...)
  return tostring(...)
end

local function mysplit(inputstr, sep)
        if sep == nil then
                sep = "%s"
        end
        local t={}
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
          table.insert(t, str)                
        end
        return t
end

local Addon = {}
Addon.initialized = false
Addon.debug = false
Addon.name = DEG_ADDON.ADDON_NAME
Addon.versionString = '1.014'
Addon.saveVariablesName = DEG_ADDON.SAVED_VARS_NAME
Addon.savedVariablesAccount = nil
Addon.saveVariablesVersion = 1
Addon.vars = {
  importDone = false,
  defaults = {
    account = {
      optRedGreenFish = true,
      optDoExport = true,
      tsVotansFishermanMerge = nil --the timestamp of the last imported fisherman merged file    
    }
  }  
}
Addon.Settings = _G[DEG_ADDON.ADDON_NAME.."Settings"]


function Addon:Initialize()
  if (self.initialized) then return end
  
  --if self.debug then
    --self.saveVariablesVersion = GetTimeStamp()
  --end
    
  self.savedVariablesAccount = ZO_SavedVars:NewAccountWide(self.saveVariablesName, self.saveVariablesVersion, nil, self.vars.defaults.account)
        
        
  self.libSafe = LibStub("LibSafe").new(self, self.name, self.saveVariablesName, self.debug)
  self.libSafe:makeTableSafe()
  self.libSafe:makeTableSafe(self.Settings)
  
  EVENT_MANAGER:RegisterForEvent(self.name, EVENT_PLAYER_ACTIVATED, function(...) self:onEVENT_PLAYER_ACTIVATED(...) end)  
  EVENT_MANAGER:RegisterForEvent(self.name, EVENT_PLAYER_DEACTIVATED, function(...) self:onEVENT_PLAYER_DEACTIVATED(...) end) 
    
  local moreMotd = ""
  if self.debug then moreMotd =" ,"..ts(GetGameTimeMilliseconds()) end  
  LibStub("LibMOTD"):setMessage(self.savedVariablesAccount, "|c3f95ffDryzler's|r |cEFEBBEFisherman|r: "..GetString(SI_DEG_FISHERMAN_MOTD)..moreMotd, 1) 
  
  
  local votansControl = RETICLE.interact:GetNamedChild("Fisherman")
  
  if votansControl then  
    ZO_PreHook(votansControl, "SetHidden", function(setHiddenUd, setHidden)          
      if setHidden then
        self:hideFishingInfo()
      else
        d("votansControl.SetHidden "..ts(setHidden))
        self:showFishingInfo()
      end
    end)  
  end
   
  self:doVotansFishermanMergeImport()
   
  self.Settings:initialize()

  self.initialized = true
end


function Addon:doVotansFishermanMergeExport()
  if not self.savedVariablesAccount.optDoExport then return end

  if not DryzlersFishermanExportSavedVars then
    d("doVotansFishermanMergeExport> not DryzlersFishermanExportSavedVars")
    return
  end
  
  if not VOTANS_FISHERMAN then 
    d("doVotansFishermanMergeExport> not VOTANS_FISHERMAN")
    return 
  end
  if not VOTANS_FISHERMAN.settings then
    d("doVotansFishermanMergeExport> not VOTANS_FISHERMAN.settings")
    return 
  end  
  if not VOTANS_FISHERMAN.settings.zoneToLure4 then
    d("doVotansFishermanMergeExport> not VOTANS_FISHERMAN.settings.zoneToLure4")
    return 
  end      
  
  d("doVotansFishermanMergeExport> saving VOTANS_FISHERMAN.settings.zoneToLure4")
  DryzlersFishermanExportSavedVars = VOTANS_FISHERMAN.settings.zoneToLure4
end

function Addon:doVotansFishermanMergeImport()
  if not DryzlersFishermanImportSavedVars then 
    d("initializeVotansFishermanMergeImport> not DryzlersFishermanImportSavedVars")
    return 
  end
  if not DryzlersFishermanImportSavedVars.timestamp then 
    d("initializeVotansFishermanMergeImport> not DryzlersFishermanImportSavedVars.timestamp")
    return 
  end
  
  if DryzlersFishermanImportSavedVars.timestamp == 0 then
    d("initializeVotansFishermanMergeImport> DryzlersFishermanImportSavedVars.timestamp(2) == 0")
    return
  end
  
  if not DryzlersFishermanImportSavedVars.zoneToLure then 
    d("initializeVotansFishermanMergeImport> not DryzlersFishermanImportSavedVars.zoneToLure")
    return 
  end  
  
  if self.savedVariablesAccount.tsVotansFishermanMerge and self.savedVariablesAccount.tsVotansFishermanMerge >= DryzlersFishermanImportSavedVars.timestamp then
    d("initializeVotansFishermanMergeImport> self.savedVariablesAccount.tsVotansFishermanMerge="..ts(self.savedVariablesAccount.tsVotansFishermanMerge)..">="..ts(DryzlersFishermanImportSavedVars.timestamp).."=DryzlersFishermanImportSavedVars.timestamp")
    return
  end
  
  d("initializeVotansFishermanMergeImport> self.savedVariablesAccount.tsVotansFishermanMerge="..ts(self.savedVariablesAccount.tsVotansFishermanMerge))
  d("initializeVotansFishermanMergeImport> DryzlersFishermanImportSavedVars.timestamp="..ts(DryzlersFishermanImportSavedVars.timestamp))
  
  if not VOTANS_FISHERMAN then 
    d("initializeVotansFishermanMergeImport> not VOTANS_FISHERMAN")
    return 
  end
  if not VOTANS_FISHERMAN.settings then
    d("initializeVotansFishermanMergeImport> not VOTANS_FISHERMAN.settings")
    return 
  end  
  if not VOTANS_FISHERMAN.settings.zoneToLure4 then
    d("initializeVotansFishermanMergeImport> not VOTANS_FISHERMAN.settings.zoneToLure4")
    return 
  end    
  
  --backup first import zone2Lure
  if not self.savedVariablesAccount.backupFirstZone2Lure then
    d("initializeVotansFishermanMergeImport> backup initial VOTANS_FISHERMAN.settings.zoneToLure4")
    self.savedVariablesAccount.backupFirstZone2Lure = VOTANS_FISHERMAN.settings.zoneToLure4
  end
  
  --backup last import zone2Lure
  d("initializeVotansFishermanMergeImport> backup last VOTANS_FISHERMAN.settings.zoneToLure4")
  self.savedVariablesAccount.backupLastZone2Lure = VOTANS_FISHERMAN.settings.zoneToLure4  
    
  d("initializeVotansFishermanMergeImport> setting VOTANS_FISHERMAN.settings.zoneToLure4")
  VOTANS_FISHERMAN.settings.zoneToLure4 = DryzlersFishermanImportSavedVars.zoneToLure

  self.savedVariablesAccount.tsVotansFishermanMerge = DryzlersFishermanImportSavedVars.timestamp
  
  self.vars.importDone = true   
end

function Addon:restoreInitialFishermanData()
  if not self.savedVariablesAccount.backupFirstZone2Lure then return end
  
  if not VOTANS_FISHERMAN then 
    d("initializeVotansFishermanMergeImport> not VOTANS_FISHERMAN")
    return 
  end
  if not VOTANS_FISHERMAN.settings then
    d("initializeVotansFishermanMergeImport> not VOTANS_FISHERMAN.settings")
    return 
  end
  
  VOTANS_FISHERMAN.settings.zoneToLure4 = self.savedVariablesAccount.backupFirstZone2Lure
  
  ReloadUI()
  
  --self.savedVariablesAccount.backupFirstZone2Lure = nil
end

function Addon:restoreLastFishermanData()
  if not self.savedVariablesAccount.backupLastZone2Lure then return end
  
  if not VOTANS_FISHERMAN then 
    d("initializeVotansFishermanMergeImport> not VOTANS_FISHERMAN")
    return 
  end
  if not VOTANS_FISHERMAN.settings then
    d("initializeVotansFishermanMergeImport> not VOTANS_FISHERMAN.settings")
    return 
  end
  
  VOTANS_FISHERMAN.settings.zoneToLure4 = self.savedVariablesAccount.backupLastZone2Lure
  
  ReloadUI()  
end

function Addon:showFishingInfo()
  if not self.savedVariablesAccount.optRedGreenFish then return end
  
  local thisControl = RETICLE.interact:GetNamedChild("DryzlersFisherman")
  if not thisControl then
    thisControl = WINDOW_MANAGER:CreateControl("$(parent)DryzlersFisherman", RETICLE.interact, CT_TEXTURE)
    thisControl:SetTexture([[esoui/art/icons/crafting_fishing_perch.dds]])    
    thisControl:SetAnchor(LEFT, RETICLE.interactContext, LEFT, -34, 0)
    thisControl:SetDimensions(32, 32)    
    local r,g,b,a = thisControl:GetColor()
    thisControl.origR = r 
    thisControl.origG = r
    thisControl.origB = r
    thisControl.origA = r
  end
  
  thisControl:SetColor(thisControl.origR,thisControl.origG,thisControl.origB,thisControl.origA)

  local text = VOTANS_FISHERMAN.interactInfo:GetText()
    
  if text == "" then
    
  else
    --Votans Fischerman kennt mindestens einen GewässerTyp
    
    
    local tumpelTypes = mysplit(text, ",")
    --split votans text anhand ,
    
    d("anzahl tumpeltypen="..ts(#tumpelTypes))

      for k,v in pairs(tumpelTypes) do
        d("tumpelType k="..ts(k).."="..ts(v))
      end
    
    if #tumpelTypes == 1 then
    
      local tumpelType = string.gsub(tumpelTypes[1],": ","")
      
      d("tumpelType ="..ts(tumpelType))
      --genau 1 gewässertyp bekannt
    --rot
      thisControl:SetColor(1,0,0,1)
      

      
      --übersetze votans wassertyp zu RFT-Wassertyp   
      
      local rftActiveWaterType = SI_RARE_FISH_TRACKER_TYPE_OCEAN
      if tumpelType == GetString(SI_FISHERMAN_INTERACT3) then
        rftActiveWaterType = SI_RARE_FISH_TRACKER_TYPE_LAKE
      elseif tumpelType == GetString(SI_FISHERMAN_INTERACT2) then
        rftActiveWaterType = SI_RARE_FISH_TRACKER_TYPE_RIVER
      elseif tumpelType == GetString(SI_FISHERMAN_INTERACT1) then
        rftActiveWaterType = SI_RARE_FISH_TRACKER_TYPE_FOUL
      end
      
      d("rftActiveWaterType="..ts(GetString(rftActiveWaterType)))
         
      local achievements = RFT.GetAchievementsByZoneIndex(RFT.zone)
      if not achievements or achievements == 0 then 
        d("not achievements??")
        return
      end
      
      for k,v in pairs(achievements) do
        d("achievements k="..ts(k).."="..ts(v))
      end
    
      -- =0 oder table mit 1 => achievement id     
      local achievement = achievements[1]
      d("achievement ="..ts(achievement))

      local types = RFT.types[RFT.zone] or RFT.types[0]
       
      local caughtComplete = 0
      local fishes = RFT.fishnames[achievement]
      local fishquality = RFT.quality[achievement] or RFT.quality[0]
      local fishorder = RFT.orders[achievement] or RFT.orders[0]
      local countFishes = 0
      for index, fishName in ipairs(fishes) do
        --welcher tümpelTyp hat der Fisch?
        
        local order = fishorder[index]
        if order then

          local fishWaterType = SI_RARE_FISH_TRACKER_TYPE_OCEAN
          if order == RFT.window.column2 then
            fishWaterType = SI_RARE_FISH_TRACKER_TYPE_LAKE
          elseif order == RFT.window.column3 then
            fishWaterType = SI_RARE_FISH_TRACKER_TYPE_RIVER
          elseif order == RFT.window.column4 then
            fishWaterType = SI_RARE_FISH_TRACKER_TYPE_FOUL
          end          
          
          d("fishindex="..ts(index).." order="..ts(order).. "fishWaterType="..GetString(fishWaterType).." fish="..ts(fishName))
          
          if fishWaterType == rftActiveWaterType then
            if (fishquality[index] == ITEM_QUALITY_ARCANE) then
              d("caught = "..ts(RFT.progress[achievement][fishName])) 
              local caught = RFT.progress[achievement][fishName] == 1
              if caught then        
                caughtComplete = caughtComplete + 1
              end
              countFishes = countFishes + 1
            end        
            if (fishquality[index] == ITEM_QUALITY_MAGIC) then
              d("caught = "..ts(RFT.progress[achievement][fishName])) 
              local caught = RFT.progress[achievement][fishName] == 1
              if caught then
                caughtComplete = caughtComplete + 1
              end        
              countFishes = countFishes + 1
            end
            if (fishquality[index] == ITEM_QUALITY_ARTIFACT) then
              d("caught = "..ts(RFT.progress[achievement][fishName])) 
              local caught = RFT.progress[achievement][fishName] == 1
              if caught then
                caughtComplete = caughtComplete + 1
              end
              countFishes = countFishes + 1
            end                  
          end        
        else
          d("fishindex="..ts(index).." no order="..ts(order))
        end           
      end
      
    d("countFishes="..ts(countFishes).." caughtComplete="..ts(caughtComplete))
    
      if countFishes > 0 and countFishes == caughtComplete then
        thisControl:SetColor(0,0.80,0,1)
      end
    end
  end
    
  thisControl:SetHidden(false)
end

function Addon:hideFishingInfo()
  local thisControl = RETICLE.interact:GetNamedChild("DryzlersFisherman")
  if thisControl then
    thisControl:SetHidden(true)
  end
end

--#################################################################################################

function Addon:onEVENT_PLAYER_DEACTIVATED(intEventCode)
  d("onEVENT_PLAYER_DEACTIVATED: "..GetUnitName("player"))

  self:doVotansFishermanMergeExport()
end

function Addon:onEVENT_PLAYER_ACTIVATED(intEventCode, bInitial)
  d("onEVENT_PLAYER_ACTIVATED: "..GetUnitName("player"))
  
  if self.vars.importDone then
    self.vars.importDone = false

    zo_callLater(function()
      ReloadUI()
    end, 1)
    
    return    
  end
  
  self:doVotansFishermanMergeExport()
  
end

--#################################################################################################

function Addon:d(m)
  if self.debug then
    _G.d(self.name.."> "..tostring(m))
  end
end

_G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT] = Addon;

EVENT_MANAGER:RegisterForEvent(DEG_ADDON.ADDON_NAME, EVENT_ADD_ON_LOADED, 
  function(event, AddonName)
    if AddonName == _G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT].name then
      _G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT]:Initialize()
      EVENT_MANAGER:UnregisterForEvent(_G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT].name, EVENT_ADD_ON_LOADED)
    end
  end
)