local DEG_ADDON = _G["DEG_CURRENT_ADDON"]

local function d(...)
  _G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT]:d(...)
end

local LAM2 = LibStub:GetLibrary("LibAddonMenu-2.0")

local Obj = {
  initialized = false,
  Addon = nil,
}

function Obj:initialize()
  if self.initialized then return end
  
  self.Addon = _G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT]
              
  local optionsPanelConfig  = {
    type = "panel",
    name = "Dryzler's Fisherman",
    displayName = "|c3f95ffDryzler's|r |cEFEBBEFisherman|r",
    author = "|cEFEBBEdryzler.elder-geek.net|r",
    website = "https://dryzler.elder-geek.net/",
    version = self.Addon.versionString,
    slashCommand = "/dryfish",
    registerForRefresh = true,
    registerForDefaults = true,
  }

  local optionsPanel = LAM2:RegisterAddonPanel(self.Addon.name, optionsPanelConfig)
  
  local optionsPanelControls = {}
    
  table.insert(optionsPanelControls, {
      type = "header",
      name = GetString(SI_DEG_SI_SETTINGS_ACCOUNTWIDE),
  })  
    
 
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_FISH_OPTS_REDGREENFISH),
      default = self.Addon.vars.defaults.account.optRedGreenFish,
      getFunc = function()
        return self.Addon.savedVariablesAccount.optRedGreenFish 
      end,
      setFunc = function(newValue)        
        self.Addon.savedVariablesAccount.optRedGreenFish = newValue
        if newValue then
          --self.Addon:activateRedGreenFish()
        else
          --self.Addon:deactivateRedGreenFish()
        end
      end,
  })
  
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_FISH_OPTS_DOEXPORT),
      default = self.Addon.vars.defaults.account.optDoExport,
      getFunc = function()
        return self.Addon.savedVariablesAccount.optDoExport 
      end,
      setFunc = function(newValue)
        self.Addon.savedVariablesAccount.optDoExport = newValue
      end,
  })  

  table.insert(optionsPanelControls, {
      type = "divider"
  })

  table.insert(optionsPanelControls, {
    type = "description",
    title = nil,
    text = GetString(SI_DEG_FISH_OPTS_DOEXPORT_DESCRIPTION),
    width = "full",
  })

  table.insert(optionsPanelControls, {
    type = "button",
    name = GetString(SI_DEG_FISH_OPTS_DOEXPORT_BUTTON),
    func = function() RequestOpenUnsafeURL("https://dryzler.elder-geek.net/teso-fisherman-merge/") end,
    width = "full",
  })
  
  if self.Addon.savedVariablesAccount.backupFirstZone2Lure or self.Addon.savedVariablesAccount.backupLastZone2Lure then
    table.insert(optionsPanelControls, {
        type = "divider"
    })
    
    table.insert(optionsPanelControls, {
        type = "header",
        name = GetString(SI_DEG_FISH_OPTS_RESTORE_HEADER),
    })        
    
    if self.Addon.savedVariablesAccount.backupLastZone2Lure then
      table.insert(optionsPanelControls, {
        type = "button",
        name = GetString(SI_DEG_FISH_OPTS_DORESTORELAST),
        func = function()
          self:ShowOkCancel(function()
            self.Addon:restoreLastFishermanData()
          end)
        end,
        width = "full",
      })    
    end
    
    if self.Addon.savedVariablesAccount.backupFirstZone2Lure then     
      table.insert(optionsPanelControls, {
        type = "button",
        name = GetString(SI_DEG_FISH_OPTS_DORESTOREFIRST),
        func = function()
          self:ShowOkCancel(function()
            self.Addon:restoreInitialFishermanData()
          end)
        end,
        width = "full",
      })
    end
  end
    
  LAM2:RegisterOptionControls(self.Addon.name, optionsPanelControls)
  
  self.initialized = true
end

function Obj:ShowOkCancel(fn)
  ZO_Dialogs_ShowDialog(
      {
          canQueue = true,
          title = {
              text = GetString(SI_DEG_FISH_OPTS_ARE_YOU_SURE),
          },
          mainText = {
              text = GetString(SI_DEG_FISH_OPTS_ARE_YOU_SURE_DETAILS),
          },
          buttons = {
              [1] = {
                  text = GetString(SI_DEG_FISH_OPTS_ARE_YOU_SURE_YES),
                  callback = function() 
                    fn();
                  end,
              },
              [2] = {
                  text = GetString(SI_DEG_FISH_OPTS_ARE_YOU_SURE_NO),
                  callback = function() end,
              }
          },
          noChoiceCallback = function() end,
      })
end

_G[_G["DEG_CURRENT_ADDON"].ADDON_NAME.."Settings"] = Obj